@@ -26,10 +26,18 @@ extern int ARMCI_Default_Proc_Group;
 /* for armci_msg_sel_scope */
 static MPI_Datatype MPI_LONGLONG_INT;
 
+/* Information on nodes */
+extern int _number_of_procs_per_node;
+extern int _my_node_id;
+extern ARMCI_Group ARMCI_Node_group;
+
 static MPI_Comm wc()
 {
     MPI_Comm comm;
+    /*
     assert(COMEX_SUCCESS == comex_group_comm(COMEX_GROUP_WORLD, &comm));
+    */
+    comex_group_comm(COMEX_GROUP_WORLD, &comm);
     return comm;
 }
 
@@ -37,7 +45,10 @@ static MPI_Comm wc()
 MPI_Comm armci_group_comm(ARMCI_Group *group)
 {
     MPI_Comm comm;
+    /*
     assert(COMEX_SUCCESS == comex_group_comm(*group, &comm));
+    */
+    comex_group_comm(*group, &comm);
     return comm;
 }
 
@@ -94,6 +105,19 @@ static MPI_Op armci_op_to_mpi_op(char *op)
     else if (strncmp(op, "or", 2) == 0) {
         result = MPI_BOR;
     }
+    /* these are new */
+    else if ((strncmp(op, "&&", 2) == 0) || (strncmp(op, "land", 4) == 0)) {
+        result = MPI_LAND;
+    }
+    else if ((strncmp(op, "||", 2) == 0) || (strncmp(op, "lor", 3) == 0)) {
+        result = MPI_LOR;
+    }
+    else if ((strncmp(op, "&", 1) == 0) || (strncmp(op, "band", 4) == 0)) {
+        result = MPI_BAND;
+    }
+    else if ((strncmp(op, "|", 1) == 0) || (strncmp(op, "bor", 3) == 0)) {
+        result = MPI_BOR;
+    }
     else {
         printf("Unsupported gop operation:%s\n",op);
         assert(0);
@@ -133,7 +157,10 @@ static void do_abs(void *x, int n, int type)
 static MPI_Comm get_comm(ARMCI_Group *group)
 {
     MPI_Comm comm;
+    /*
     assert(COMEX_SUCCESS == comex_group_comm(*group, &comm));
+    */
+    comex_group_comm(*group, &comm);
     return comm;
 }
 
@@ -236,7 +263,11 @@ void armci_msg_sel_scope(int scope, void *x, int n, char* op, int type, int cont
     MPI_Comm comm = get_default_comm();
 
     if (SCOPE_NODE == scope) {
+      if (_number_of_procs_per_node == 1) {
         comm = MPI_COMM_SELF;
+      } else {
+        comex_group_comm(ARMCI_Node_group,&comm);
+      }
     }
 
     /* first time this function is called we establish the
@@ -299,8 +330,10 @@ void armci_msg_bcast_scope(int scope, void* buffer, int len, int root)
         armci_msg_bcast(buffer, len, root);
     }
     else if (SCOPE_NODE == scope) {
+        MPI_Comm comm;
         assert(buffer != NULL);
-        MPI_Bcast(buffer, len, MPI_BYTE, root, MPI_COMM_SELF);
+        comex_group_comm(ARMCI_Node_group,&comm);
+        MPI_Bcast(buffer, len, MPI_BYTE, root, comm);
     }
     else {
         assert(0);
@@ -437,7 +470,11 @@ void armci_msg_reduce(void *x, int n, char *op, int type)
 void armci_msg_reduce_scope(int scope, void *x, int n, char *op, int type)
 {
     if (SCOPE_NODE == scope) {
+      if (_number_of_procs_per_node > 1) {
+        do_gop(x, n, op, type, ARMCI_Node_group);
+      } else {
         do_gop(x, n, op, type, ARMCI_GROUP_SELF);
+      }
     } else {
         do_gop(x, n, op, type, get_default_group());
     }
@@ -447,7 +484,11 @@ void armci_msg_reduce_scope(int scope, void *x, int n, char *op, int type)
 void armci_msg_gop_scope(int scope, void *x, int n, char* op, int type)
 {
     if (SCOPE_NODE == scope) {
+      if (_number_of_procs_per_node > 1) {
+        do_gop(x, n, op, type, ARMCI_Node_group);
+      } else {
         do_gop(x, n, op, type, ARMCI_GROUP_SELF);
+      }
     } else {
         do_gop(x, n, op, type, get_default_group());
     }
@@ -683,7 +724,13 @@ void parmci_msg_group_barrier(ARMCI_Group *group)
 void armci_msg_group_bcast_scope(int scope, void *buf, int len, int root, ARMCI_Group *group)
 {
     if (SCOPE_NODE == scope) {
+      if (_number_of_procs_per_node > 1) {
+        MPI_Comm comm;
+        comex_group_comm(ARMCI_Node_group, &comm);
+        MPI_Bcast(buf, len, MPI_BYTE, root, comm);
+      } else {
         MPI_Bcast(buf, len, MPI_BYTE, root, MPI_COMM_SELF);
+      }
     }
     else {
         int root_sub;
